"""Utilities for testing."""

from pathlib import Path
from typing import Union

from bs4 import BeautifulSoup


def read_as_html(filename: Union[Path, str]) -> BeautifulSoup:
    """Read a test file and parse it as HTML."""
    with open(filename) as file_handle:
        tree = BeautifulSoup(file_handle, "html.parser")
    return tree



"""Sphinx configuration."""

import os

from dotenv import load_dotenv

# Load environment variables
load_dotenv()

# Project information
project = "Algolia DocSearch Sphinx extension"
author = "Algolia"
copyright = f"{author}."
html_title = "Algolia DocSearch for Sphinx"

# General configuration
extensions = ["sphinx_docsearch", "myst_parser"]
myst_enable_extensions = ["colon_fence", "deflist"]
linkcheck_ignore = ["https://dashboard.algolia.com"]
html_show_sourcelink = False

# DocSearch Sphinx extension
docsearch_app_id = os.getenv("DOCSEARCH_APP_ID")
docsearch_api_key = os.getenv("DOCSEARCH_API_KEY")
docsearch_index_name = os.getenv("DOCSEARCH_INDEX_NAME")

# Support building the docs with different themes
# Add `-t alabaster` or `-t rtd` to the build arguments
if tags.has("rtd"):  # type: ignore  # noqa
    html_theme = "sphinx_rtd_theme"
    html_theme_options = {
        "style_external_links": True,
    }
elif tags.has("alabaster"):  # noqa  # type:ignore
    html_theme = "alabaster"
elif tags.has("furo"):  # noqa  # type: ignore
    html_theme = "furo"
elif tags.has("pydata"):  # noqa  # type: ignore
    html_theme = "pydata_sphinx_theme"
else:
    from sphinxawesome_theme.postprocess import Icons

    html_theme = "sphinxawesome_theme"
    html_permalinks_icon = Icons.permalinks_icon
    html_theme_options = {
        "awesome_external_links": True,
        "extra_header_link_icons": {
            "repository on GitHub": {
                "link": "https://github.com/algolia/sphinx-docsearch",
                "icon": (
                    '<svg height="26px" style="margin-top:-2px;display:inline" '
                    'viewBox="0 0 45 44" '
                    'fill="currentColor" xmlns="http://www.w3.org/2000/svg">'
                    '<path fill-rule="evenodd" clip-rule="evenodd" '
                    'd="M22.477.927C10.485.927.76 10.65.76 22.647c0 9.596 6.223 17.736 '
                    "14.853 20.608 1.087.2 1.483-.47 1.483-1.047 "
                    "0-.516-.019-1.881-.03-3.693-6.04 "
                    "1.312-7.315-2.912-7.315-2.912-.988-2.51-2.412-3.178-2.412-3.178-1.972-1.346.149-1.32.149-1.32 "  # noqa
                    "2.18.154 3.327 2.24 3.327 2.24 1.937 3.318 5.084 2.36 6.321 "
                    "1.803.197-1.403.759-2.36 "
                    "1.379-2.903-4.823-.548-9.894-2.412-9.894-10.734 "
                    "0-2.37.847-4.31 2.236-5.828-.224-.55-.969-2.759.214-5.748 0 0 "
                    "1.822-.584 5.972 2.226 "
                    "1.732-.482 3.59-.722 5.437-.732 1.845.01 3.703.25 5.437.732 "
                    "4.147-2.81 5.967-2.226 "
                    "5.967-2.226 1.185 2.99.44 5.198.217 5.748 1.392 1.517 2.232 3.457 "
                    "2.232 5.828 0 "
                    "8.344-5.078 10.18-9.916 10.717.779.67 1.474 1.996 1.474 4.021 0 "
                    "2.904-.027 5.247-.027 "
                    "5.96 0 .58.392 1.256 1.493 1.044C37.981 40.375 44.2 32.24 44.2 "
                    '22.647c0-11.996-9.726-21.72-21.722-21.72" '
                    'fill="currentColor"/></svg>'
                ),
            },
        },
    }



"""Test builder."""

import os
from pathlib import Path

import pytest
from sphinx.application import Sphinx


@pytest.mark.sphinx("html", confoverrides={"extensions": ["sphinx_docsearch"]})
def test_no_builtin_search(app: Sphinx) -> None:
    """It doesn't add the builtin search assets to the HTML output."""
    app.build()
    assert app.builder.search is False  # type: ignore
    assert not os.path.exists(Path(app.outdir) / "search.html")
    assert not os.path.exists(Path(app.outdir) / "searchindex.js")


@pytest.mark.sphinx("xml", confoverrides={"extensions": ["sphinx_docsearch"]})
def test_xml_docs(app: Sphinx) -> None:
    """It builds docs other than HTML."""
    app.build()
    assert os.path.exists(Path(app.outdir) / "index.xml")



"""Configuration for Pytest."""

from typing import Any

import pytest
from _pytest.config import Config
from sphinx import version_info

if version_info >= (7, 2):  # type: ignore
    from pathlib import Path
else:
    from sphinx.testing.path import path

pytest_plugins = "sphinx.testing.fixtures"
collect_ignore = ["roots"]


@pytest.fixture(scope="session")
def rootdir() -> Any:  # noqa: ANN401
    """Root directory for test files."""
    if version_info >= (7, 2):  # type: ignore
        return Path(__file__).parent.absolute() / "roots"  # type: ignore
    else:
        return path(__file__).parent.abspath() / "roots"  # type: ignore


def pytest_configure(config: Config) -> None:
    """Register `sphinx` marker with pytest."""
    config.addinivalue_line("markers", "sphinx")



"""Test configuration checks."""

import os
from io import StringIO
from pathlib import Path

import pytest
from sphinx.application import Sphinx

import sphinx_docsearch


def test_returns_version() -> None:
    """It returns the correct version."""
    assert sphinx_docsearch.__version__ == "0.2.0b2"


@pytest.mark.sphinx("html", confoverrides={"extensions": ["sphinx_docsearch"]})
def test_docsearch_config(app: Sphinx) -> None:
    """It tests the addition of DocSearch configuration options."""
    app.build()
    assert os.path.exists(Path(app.outdir) / "index.html")
    assert "sphinx_docsearch" in app.extensions
    assert not app.config.docsearch_app_id
    assert not app.config.docsearch_api_key
    assert not app.config.docsearch_index_name
    assert app.config.docsearch_container == "#docsearch"
    assert not app.config.docsearch_initial_query
    assert not app.config.docsearch_placeholder
    assert not app.config.docsearch_search_parameters
    assert not app.config.docsearch_max_results_per_group
    assert not app.config.docsearch_missing_results_url


@pytest.mark.sphinx("html", confoverrides={"extensions": ["sphinx_docsearch"]})
def test_raises_warnings(app: Sphinx, warning: StringIO) -> None:
    """It raises warnings when the required config options are missing."""
    app.build()
    warnings = warning.getvalue()

    assert (
        "WARNING: You must provide your Algolia application ID for DocSearch to work."
        in warnings
    )
    assert (
        "WARNING: You must provide your Algolia search API key for DocSearch to work."
        in warnings
    )
    assert (
        "WARNING: You must provide your Algolia index name for DocSearch to work."
        in warnings
    )


@pytest.mark.sphinx(
    "html",
    confoverrides={
        "extensions": ["sphinx_docsearch"],
        "docsearch_app_id": "test_app_id",
        "docsearch_api_key": "test_api_key",
        "docsearch_index_name": "test_docsearch_index_name",
        "docsearch_container": "test_docsearch_container",
        "docsearch_placeholder": "test_docsearch_placeholder",
        "docsearch_initial_query": "test_docsearch_initial_query",
        "docsearch_search_parameters": {"hitsPerPage": 5},
        "docsearch_max_results_per_group": 10,
        "docsearch_missing_results_url": "test_docsearch_url",
    },
)
def test_add_docsearch_config(app: Sphinx, warning: StringIO) -> None:
    """It parses the DocSearch configuration options."""
    app.build()
    warnings = warning.getvalue()
    assert warnings == ""
    assert app.config.docsearch_app_id == "test_app_id"
    assert app.config.docsearch_api_key == "test_api_key"
    assert app.config.docsearch_index_name == "test_docsearch_index_name"
    assert app.config.docsearch_container == "test_docsearch_container"
    assert app.config.docsearch_placeholder == "test_docsearch_placeholder"
    assert app.config.docsearch_initial_query == "test_docsearch_initial_query"
    assert app.config.docsearch_search_parameters == {"hitsPerPage": 5}
    assert app.config.docsearch_max_results_per_group == 10
    assert app.config.docsearch_missing_results_url == "test_docsearch_url"



# /// script
# dependencies = [
#     "httpx",
# ]
# ///

"""Update DocSearch assets from jsDelivr.

This extension bundles the JavaScript and CSS for DocSearch.
This module downloads the latest version of these assets from
the jsDelivr CDN.
"""

import re
from pathlib import Path

import httpx


def update_docsearch_assets() -> None:
    """Update the DocSearch assets."""
    for package in ["@docsearch/js", "@docsearch/css"]:
        _download_asset(package)


def _download_asset(package: str) -> None:
    """Download an asset from jsDelivr."""
    cdn_uri = (
        f"https://cdn.jsdelivr.net/npm/{package}@latest/{_jsdelivr_file_name(package)}"
    )
    response = httpx.get(cdn_uri)
    output = package.replace("@", "").replace("/", ".")
    output_path = Path("src/sphinx_docsearch/static/").resolve()
    with open(Path(output_path / output).absolute(), "w") as f:
        f.write(re.sub(r"//#.*\n$", "", response.text))


def _jsdelivr_file_name(package: str) -> str:
    """Get the filename for an asset on jsDelivr from NPM."""
    npm_uri = f"https://registry.npmjs.org/{package}/latest"
    response = httpx.get(npm_uri)
    return response.json()["jsdelivr"]


if __name__ == "__main__":
    update_docsearch_assets()



"""Test the DOM."""

from pathlib import Path

import pytest
from sphinx.application import Sphinx

from .utils import read_as_html


@pytest.mark.sphinx(
    "html",
    confoverrides={
        "extensions": ["sphinx_docsearch"],
        "docsearch_container": "docsearch-test",
    },
)
def test_no_builtin_search(app: Sphinx) -> None:
    """It adds the docsearch container to the DOM."""
    app.build()

    test_file = read_as_html(Path(app.outdir) / "index.html")
    search = test_file.select("#docsearch-test")
    assert len(search) == 1



"""Automate everything everywhere all at once.

Requires [Nox](https://nox.thea.codes/).

- Run `nox -ls` to list all sessions.
- Run `nox -s <NAME>` to run the session _`<NAME>`_.
- Run `nox -s docs -p 3.13 -- --live` to build the docs with live-reloading.
"""

from __future__ import annotations

import nox

nox.options.stop_on_first_error = True
nox.options.default_venv_backend = "uv"
nox.options.sessions = ["tests", "typecheck"]

python_versions = ["3.9", "3.13"]


def get_requirements(groups: list[str] | str | None = None) -> list[str]:
    """Load requirements from a `pyproject.toml` file."""
    pyproject = nox.project.load_toml("pyproject.toml")
    pkgs = pyproject["project"]["dependencies"]

    if groups and "dependency-groups" in pyproject:
        for g in groups if isinstance(groups, list) else [groups]:
            pkgs += pyproject["dependency-groups"].get(g, [])

    return pkgs


def install_requirements(
    session: nox.Session, groups: list[str] | str | None = None
) -> None:
    """Install requirements into the session's environment."""
    requirements = get_requirements(groups)
    session.install(*requirements)
    session.install("-e", ".")


@nox.session(python=python_versions)
def docs(s: nox.Session) -> None:
    """Build the docs."""
    args = ["-aWTE", "docs", "docs/_dist"]
    sphinx_build = "sphinx-build"

    if "--live" in s.posargs:
        sphinx_build = "sphinx-autobuild"
        s.posargs.remove("--live")

    if s.posargs:
        args = s.posargs + args

    install_requirements(s, "docs")
    s.run(sphinx_build, *args)


@nox.session
def check_links(s: nox.Session) -> None:
    """Check links in docs."""
    args = ["-b", "linkcheck", "docs", "docs/_dist/_links"]

    if s.posargs:
        args = s.posargs + args

    install_requirements(s, "docs")
    s.run("sphinx-build", *args)


@nox.session
def fmt(s: nox.Session) -> None:
    """Format the code with ruff."""
    install_requirements(s, "lint")
    s.run("ruff", "check", ".", "--select", "I", "--fix")
    s.run("ruff", "format", ".")


@nox.session
def lint(s: nox.Session) -> None:
    """Lint the code with ruff."""
    install_requirements(s, "lint")
    s.run("ruff", "check", ".")


@nox.session(python=python_versions)
def tests(s: nox.Session) -> None:
    """Run unit tests."""
    args = s.posargs or ["--cov"]
    install_requirements(s, "test")
    s.run("pytest", *args)


@nox.session(python=python_versions)
def typecheck(s: nox.Session) -> None:
    """Typecheck."""
    install_requirements(s, "typecheck")
    s.run("pyright")


@nox.session(python=False)
def build(s: nox.Session) -> None:
    """Build the packages."""
    s.run("uv", "build", external=True)


@nox.session(python=False)
def export(s: nox.Session) -> None:
    """Export dependencies for ReadTheDocs."""
    s.run(
        "uv",
        "export",
        "--group=docs",
        "--no-hashes",
        "--output-file=docs/requirements.txt",
        external=True,
    )


@nox.session(python=False)
def clean(s: nox.Session) -> None:
    """Delete artifacts."""
    s.run("rm", "-rv", "dist", "docs/_dist")
